# principles and practice of sem, 5th ed.
# rex b. kline, guilford press

# chapter 11, table 11.2, analysis 1
# fit two nonnested path models by romney et al. (1992)
# to the same correlation matrix and plausible 
# standard deviations, generate AIC and BIC

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))
library(lavaan)

# get citation information
citation("lavaan", auto = TRUE)

# input the correlations in lower diagnonal form
romneyLower.cor <- '
 1.00
  .53 1.00
  .15  .18 1.00
  .52  .29 -.05 1.00
  .30  .34  .23  .09 1.00 '

# name the variables and convert to full correlation matrix
romney.cor <- lavaan::getCov(romneyLower.cor, names = c("morale", "symptoms",
 "dysfunction", "relations", "ses"))

# display the correlations
romney.cor

# add the standard deviations and convert to covariances
romney.cov <- lavaan::cor2cov(romney.cor, sds = c(3.75,17.00,19.50,
 3.50,24.70))

# display the covariances
romney.cov

# specify psychosomatic path model

somatic.model <- '
# regressions
 morale ~ ses + dysfunction
 relations ~ morale
 symptoms ~ morale
# without the zero constraint listed next,
# lavaan automatically specifies correlated
# disturbances for symptoms and relations,
# but their disturbances are independent in
# figure 11.1
 symptoms ~~ 0*relations
# unanalyzed association between ses and dysfunction
# automatically specified '

# specify conventional medical model

medical.model <- '
# regressions
 ses ~ symptoms + dysfunction
 morale ~ symptoms + ses
 relations ~ dysfunction + morale
# unanalyzed association between symptoms and dysfunction
# automatically specified '

# fit models to sample covariance matrix, N = 469
# variances and covariance of measured exogenous
# variables are free parameters
# variances calculated with N in the denominator

somatic <- lavaan::sem(somatic.model, sample.cov = romney.cov,
 sample.nobs = 469, fixed.x = FALSE, sample.cov.rescale = FALSE)

# request output 
lavaan::summary(somatic, fit.measures = TRUE, rsquare = TRUE)

# print residuals
lavaan::residuals(somatic, type = "standardized.mplus")
lavaan::residuals(somatic, type = "normalized")
lavaan::residuals(somatic, type = "cor.bollen")

medical <- lavaan::sem(medical.model, sample.cov=romney.cov,
 sample.nobs = 469, fixed.x = FALSE, sample.cov.rescale = FALSE)

# request output 
lavaan::summary(medical, fit.measures = TRUE, rsquare = TRUE)

# print residuals
lavaan::residuals(medical, type = "standardized.mplus")
lavaan::residuals(medical, type = "normalized")
lavaan::residuals(medical, type = "cor.bollen")



