# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 14, table 14.1, analysis 1
# single-factor cfa model for the kabc-1 analyzed
# in the 10-year-old normative sample for the test

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)

# get citation information
citation("lavaan", auto = TRUE)

# input the correlations in lower diagnonal form
kabcLower.cor <- '
 1.00
 .39 1.00
 .35  .67 1.00
 .21  .11  .16 1.00
 .32  .27  .29  .38 1.00
 .40  .29  .28  .30  .47 1.00
 .39  .32  .30  .31  .42  .41 1.00
 .39  .29  .37  .42  .58  .51  .42 1.00 '

# name the variables and convert to full correlation matrix
# hm, hand movements; nr, number recall; wo, word order; gc, gestalt closure; 
# tr, triangles; sm, spatial memory; ma, matrix analogies; ps, photo series
kabc.cor <- lavaan::getCov(kabcLower.cor, names = c("hm", "nr", "wo",
 "gc", "tr", "sm", "ma", "ps"))
# display correlations
kabc.cor

# add the standard deviations and convert to covariances
kabc.cov <- lavaan::cor2cov(kabc.cor, sds = c(3.40,2.40,2.90,2.70,2.70,4.20,2.80,3.00))

# display covariances
kabc.cov

# specify 1-factor cfa model
# single factor (general ability)
# indicator hm automatically
# specified as reference variable

kabc1.model <- '
General =~ hm + nr + wo + gc + tr + sm + ma + ps '

# fit model to data
# variances calculated with N in the denominator, not N - 1
kabc1 <- lavaan::sem(kabc1.model, sample.cov = kabc.cov, sample.nobs = 200)
lavaan::summary(kabc1, fit.measures = TRUE, standardized = TRUE, rsquare = TRUE)

# residuals
lavaan::residuals(kabc1, type = "raw")
lavaan::residuals(kabc1, type = "standardized.mplus")
lavaan::residuals(kabc1, type = "cor.bollen")

