# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 14, table 14.1, analysis 3
# example of heywood cases in a basic two-factor
# model analyzed in a small sample, n = 103

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)

# get citation information
citation("lavaan", auto = TRUE)

# input the correlations in lower diagnonal form
heywoodLower.cor <- '
 1.000
  .740 1.000
  .265  .422 1.000
  .305  .401  .791 1.000
  .315  .351  .662  .587 1.000 '

# name the variables and convert to full correlation matrix
heywood.cor <- lavaan::getCov(heywoodLower.cor, names = c("problems", "intimacy",
 "father", "mother", "both"))

# display correlations
heywood.cor

# add the standard deviations and convert to covariances
heywood.cov <- lavaan::cor2cov(heywood.cor, sds = c(32.936,22.749,
 13.390,13.679,14.382))

# display covariances
heywood.cov

# specify 2-factor model
# matital adjustment and family-of-origin (foe) experiences

heywood.model <- '
Marital =~ problems + intimacy
FOE =~ father + mother + both '

# fit model to data
heywood <- lavaan::sem(heywood.model, sample.cov = heywood.cov,
 sample.nobs = 103)
lavaan::summary(heywood, fit.measures = TRUE, standardized = TRUE,
 rsquare = TRUE)

# the confidence interval for the 
# negative error variance includes zero
lavaan::parameterEstimates(heywood)


