# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 15, table 15.1, analysis 3
# single-factor cfa model for tests of the cogstate 
# schizophrenia battery 

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)

# get citation information
citation("lavaan", auto = TRUE)

# 12 csb tests

# isl, international shopping list test
# islr, international shopping list delayed recall
# oneb, one-back test
# twob, Two-Back Test
# gml, groton maze learning test
# sets, set-shifting test
# det, detection test
# gmct, groton maze chase test
# ocl, one-card learning test
# cpal, continuous paired associate learning test
# gmr, groton maze learning rask delayed recall
# idn, identification test

# input the correlations in lower diagnonal form
csbLower.cor <- '
1.000
 .753 1.000
 .238  .236 1.000
 .290  .248  .460 1.000
 .329  .334  .397  .442 1.000
 .275  .218  .351  .474  .352 1.000
 .289  .277  .309  .362  .193  .128 1.000
 .424  .398  .291  .327  .314  .127  .559 1.000
 .398  .347  .474  .369  .411  .325  .203  .326 1.000
 .430  .439  .301  .416  .526  .296  .294  .422  .470 1.000
 .316  .307  .398  .362  .672  .312  .257  .307  .451  .532 1.000
 .239  .282  .390  .281  .332  .305  .027  .147  .304  .203  .298 1.000 '

# name the variables and convert to full correlation matrix
csb.cor <- lavaan::getCov(csbLower.cor, names = c("isl","isrl","oneb",
 "twob","gml","sets","det","gmct","ocl","cpal","gmr","idn"))

# add the standard deviations and convert to covariances
csb.cov <- lavaan::cor2cov(csb.cor,sds = c(4.713,2.670,.122,.124,
 1.664,.032,.036,.436,.116,.121,1.009,.064))

# display the correlations and covariances
options(width = 150)
csb.cor
csb.cov

# indicators must share at .30% of its variance
# with the single factor, otherwise it is removed
# and the respecified model is then tested

# overview

# model 1, all 12 indicators
# removed: sets, det, idn

# model 2: 9 indicators
# removed: oneb, gmct

# model 3: 7 indicators
# removed: twob

# model 4: 6 indicators
# none removed

# model 5: 6 indicators (isl, isrl, gml, ocl, cpal, gmr)
# error correlations for pairs (isl, islr), (gml, gmr)

# model 1

csb1.model <- '
 Cognitive =~ isl + isrl + oneb + twob + gml + sets + det +
  gmct + ocl + cpal + gmr + idn '

csb1 <- lavaan::sem(csb1.model, sample.cov = csb.cov, sample.nobs = 193)
lavaan::summary(csb1, fit.measures = TRUE, standardized = TRUE,
 rsquare = TRUE)

# model 2 (removed: sets, det, idn)

csb2.model <- '
 Cognitive =~ isl + isrl + oneb + twob + gml + gmct + ocl + 
  cpal + gmr '

csb2 <- lavaan::sem(csb2.model, sample.cov = csb.cov, sample.nobs = 193)
lavaan::summary(csb2, fit.measures = TRUE, standardized = TRUE,
 rsquare = TRUE)

# model 3 (removed: sets, det, idn, oneb, gmct)

csb3.model <- '
 Cognitive =~ isl + isrl + twob + gml + ocl + cpal + gmr '

csb3 <- lavaan::sem(csb3.model, sample.cov = csb.cov, sample.nobs = 193)
lavaan::summary(csb3, fit.measures = TRUE, standardized = TRUE,
 rsquare = TRUE)

# model 4 (removed: sets, det, idn, oneb, gmct, twob)

csb4.model <- '
 Cognitive =~ isl + isrl + gml + ocl + cpal + gmr '

csb4 <- lavaan::sem(csb4.model, sample.cov = csb.cov, sample.nobs = 193)
lavaan::summary(csb4, fit.measures = TRUE, standardized = TRUE,
 rsquare = TRUE)
lavaan::modindices(csb4)

# model 5
# final indicator set (isl, isrl, gml, ocl, cpal, gmr)
# add two error covariances

csb5.model <- '
 Cognitive =~ isl + isrl + gml + ocl + cpal + gmr
 isl ~~ isrl
 gml ~~ gmr '

csb5 <- lavaan::sem(csb5.model, sample.cov =  csb.cov, sample.nobs = 193)
lavaan::summary(csb5, fit.measures = TRUE, standardized = TRUE,
 rsquare = TRUE)
lavaan::residuals(csb5, type = "raw")
lavaan::residuals(csb5, type = "standardized.mplus")
lavaan::residuals(csb5, type = "cor.bollen")
lavaan::modindices(csb5)
