# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 15, table 15.1, analysis 1
# step 1 (evaluate cfa measurement model) in 2-step
# modeling for a full SR model of achievement and
# classroom adjustment

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)
library(semTools)

# get citation information
citation("lavaan", auto = TRUE)
citation("semTools", auto = TRUE)

# input the correlations in lower diagnonal form
worlandLower.cor <- '
1.00
 .70 1.00
 .65  .60 1.00
 .55  .50  .45 1.00
 .50  .45  .40  .70 1.00
 .35  .35  .30  .55  .50 1.00 
 .30  .30  .30  .50  .45  .44 1.00
 .25  .20  .22  .41  .28  .34  .40 1.00
 .35  .32  .32  .48  .45  .42  .60  .45 1.00
-.25 -.24 -.22 -.21 -.18 -.15 -.15 -.12 -.17 1.00
-.22 -.26 -.30 -.25 -.22 -.18 -.17 -.14 -.20  .42 1.00 '

# name the variables and convert to full correlation matrix
worland.cor <- lavaan::getCov(worlandLower.cor, names = c("verbal","visual",
 "memory","read","math","spell","motive","harmony","stable","parent","ses"))

# add the standard deviations and convert to covariances
worland.cov <- lavaan::cor2cov(worland.cor, 
 sds = c(13.75,14.80,12.60,14.90,15.25,13.85,9.50,11.10,8.70,
 12.00,8.50))

options(width = 130)

# display correlations
worland.cor

# display covariances
worland.cov

# maccallum-rmsea power
# exact-fit test
# cfa model with df = 38 at step 1
# power at N = 158
semTools::findRMSEApower(0, .05, 38, 158, .05, 1)
# minimum N for power at least .90
semTools::findRMSEAsamplesize(0, .05, 38, .90, .05, 1)

# step 1
# 4-factor CFA

worlandCFA.model <- '
 Cognitive =~ verbal + visual + memory
 Achieve =~ read + math + spell
 Adjust =~ motive + harmony + stable
 Risk =~ parent + ses '

worlandCFA <- lavaan::cfa(worlandCFA.model, sample.cov = worland.cov,
 sample.nobs = 158)
lavaan::summary(worlandCFA, fit.measures = TRUE, standardized = TRUE,
 rsquare = TRUE)

# predicted covariance matrix
lavaan::fitted(worlandCFA)
# predicted correlation matrix for indicators
lavaan::lavInspect(worlandCFA, "cor.ov")
# predicted correlation matrix for factors
lavaan::lavInspect(worlandCFA, "cor.lv")

# residuals
lavaan::residuals(worlandCFA, type = "raw")
lavaan::residuals(worlandCFA, type = "standardized.mplus")
lavaan::residuals(worlandCFA, type = "cor.bollen")

# calculate factor reliability coefficients (semTools)
semTools::reliability(worlandCFA)
