# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 6, table 6.3, analysis 1
# locate all conditional independencies and generate
# a union basis set for the roth et al. (1989) 
# nonparametric model of illness

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(dagitty)
library(ggm)
library(CauseAndCorrelation)

# get citation information
citation("dagitty", auto = TRUE)
citation("ggm", auto = TRUE)
citation("CauseAndCorrelation", auto = TRUE)

# specify model in dagitty
# locate all conditional independencies
# implied by the graph

rothdag1 <- dagitty::dagitty("dag{
 exercise <-> hardy;
 exercise -> fitness; hardy -> stress;
 fitness -> illness; stress -> illness}")

dagitty::impliedConditionalIndependencies(rothdag1, type="all.pairs")

# this version of dagitty did not generate a union basis set

# specify model in ggm
# "~" means "regressed on"
# U is an unmeasured common cause 

rothdag2 <- ggm::DAG(
 fitness ~ exercise,
 stress ~ hardy,
 illness ~ fitness + stress,
 exercise ~ U, hardy ~ U)

# use CauseAndCorrelation to convert ggm dag
# to a mixed acyclic graph (mag) that omits the
# the latent variable while keeping all implied
# conditional independencies among observed variables
# generate union basis for the mag

rothmag <- CauseAndCorrelation::DAG.to.MAG(full.DAG = rothdag2,
 latents = c("U"))

CauseAndCorrelation::basiSet.mag(rothmag)
